#!/usr/bin/env python
# -*- coding: utf-8 -*-

from scipy.signal import butter, lfilter
from helper_code import *

"""
信号预处理并提取数据特征和标签
Updated on 2021/06/23 by pan xia
"""

usedClasses = ['164889003', '164890007', '6374002', '426627000', '733534002', '713427006', '270492004', '713426002',
               '39732003', '445118002', '164909002', '251146004', '698252002', '426783006', '284470004', '10370003',
               '365413008', '427172004', '164947007', '111975006', '164917005', '47665007', '59118001', '427393009',
               '426177001', '427084000', '63593006', '164934002', '59931005', '17338001']


################################################################################
# Find unique classes
################################################################################
def get_classes(header_files):
    classes = set()
    for header_file in header_files:
        header = load_header(header_file)
        for l in header.split('\n'):
            if l.startswith('#Dx:'):
                entries = l.split(': ')[1].split(',')
                for entry in entries:
                    if entry.strip() in usedClasses:  # only add used 30 classes.
                        classes |= set(entry.strip())
                    else:
                        continue
    return classes


################################################################################
# data preprocess
################################################################################
def dataPrepare(header_files, recording_files, leads, classes):
    inner_all_features = []
    inner_all_trueLabels = []
    inner_all_seglength_fs = []
    single_seglength_num = 0

    for i in range(len(recording_files)):
        # print('    {}/{}...'.format(i + 1, len(recording_files)))
        single_trueLabel = np.zeros(len(classes))

        # Load header and recording.
        header = load_header(header_files[i])
        recording = load_recording(recording_files[i])

        single_features, single_seglength_fs = get_12ECG_features(header, recording, leads)

        # 提取真实标签
        for l in header.split('\n'):
            if l.startswith('#Dx:'):
                entries = l.split(': ')[1].split(',')
                for entry in entries:
                    if entry.strip() in usedClasses:  # only add used 30 classes.
                        class_index = classes.index(entry.strip())  # Only use first positive index
                        single_trueLabel[class_index] = 1
                    else:
                        continue

        inner_all_features.append(single_features)
        inner_all_trueLabels.append(single_trueLabel)
        inner_all_seglength_fs.append(single_seglength_fs)

    return inner_all_features, inner_all_trueLabels, inner_all_seglength_fs


################################################################################
# extract features
################################################################################
def get_12ECG_features(header, recording, leads, useFilter=True):
    featureIndividual = []
    targetFreq = 100  # Downsampling Frequence

    filter_lowcut = 0.05
    filter_highcut = 35.0
    filter_order = 3
    sample_Fs = get_frequency(header)
    seglength = int(get_num_samples(header)) // sample_Fs

    # Reorder/reselect leads in recordings.
    recording = choose_leads(recording, header, leads)

    # Pre-process recordings.
    adc_gains = get_adc_gains(header, leads)
    baselines = get_baselines(header, leads)

    # for i in range(len(leads)):
    #     recording[i, :] = (recording[i, :] - baselines[i]) / adc_gains[i]

    for i in range(len(leads)):
        if useFilter:
            filtered_ecg_lead = bandpass_filter(recording[i], lowcut=filter_lowcut, highcut=filter_highcut,
                                               signal_freq=sample_Fs, filter_order=filter_order)
        else:
            filtered_ecg_lead = recording[i]

        down_filtered_ecg_lead = myDownSample(filtered_ecg_lead, sample_Fs, targetFreq)  # Downsampled to 100Hz

        res = dataLenCheck(down_filtered_ecg_lead[:1000], 1000)  # 10 s
        featureIndividual.append(res)
    return featureIndividual, [seglength, sample_Fs]


# 数据归一化
def data_uniform(totalFeatures, totalLabels):

    totalFeatures = np.array(totalFeatures, dtype='float32')  # 将64改成32，节省内存
    total_Labels = np.array(totalLabels)
    theMean = np.mean(totalFeatures).tolist()
    theStd = np.std(totalFeatures).tolist()
    uniform_total_features = (totalFeatures - theMean) / theStd

    uniform_total_features[np.isnan(uniform_total_features)] = 0.01
    uniform_total_features[np.isinf(uniform_total_features)] = 0.01

    total_features = uniform_total_features

    return total_features, total_Labels, theMean, theStd


################################################################################
# bandpass filter
################################################################################
def bandpass_filter(data, lowcut, highcut, signal_freq, filter_order):
    """
    Method responsible for creating and applying Butterworth filter.
    :param deque data: raw data
    :param float lowcut: filter lowcut frequency value
    :param float highcut: filter highcut frequency value
    :param int signal_freq: signal frequency in samples per second (Hz)
    :param int filter_order: filter order
    :return array: filtered data
    """
    nyquist_freq = 0.5 * signal_freq
    low = lowcut / nyquist_freq
    high = highcut / nyquist_freq
    b, a = butter(filter_order, [low, high], btype="band")
    y = lfilter(b, a, data)
    return y


################################################################################
# Down sampling
################################################################################
def myDownSample(data, sample_Fs, targetFreq):
    step = sample_Fs // targetFreq
    newData = [data[i] for i in range(len(data)) if i % step == 0]
    return newData


################################################################################
# data aligning
################################################################################
def dataLenCheck(data, window):
    if len(data) == window:
        return data
    elif len(data) < window:
        data += [0 for i in range(window - len(data))]  # Fill data by 0.
        return data
    else:
        raise Exception("Error in dataLenCheck")
    # 防止数据片段小于window


